;Project homepage: www.HomoFaciens.de
;Program controls a stepper motor used for an electric powered passenger car. For details visit the hompage of the project.
;Copyright (C) 2011 Norbert Heinz
;
;This program is free software you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation version 3 of the License.
;This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
;You should have received a copy of the GNU General Public License along with this program if not, see http://www.gnu.org/licenses/.

;Version 0.1 - 12 electromagnets at stator, 4 permanentmagnets at rotor

.include "m8def.inc"

.def temp01  = r16
.def temp02  = r17
.def StepTimeCount = r18
.def TimerPause = r19
.def ControlBits = r20         ;Bit 0 - forward/reverse direction
                               ;Bit 1 - forward/reverse switch pressed
                               ;Bit 2 - active channel for ADC
                               ;Bit 3 + 4 - half/quater usage of inductors
.def SwitchTimer = r21
.def SpeedValue = r22
.def ThrottleValue = r23


.org 0x0000
        rjmp    main                    ;Reset Handler

.org OVF0addr
        rjmp    timer0_overflow         ;Timer Overflow Handler

main:
        ldi     temp01, LOW(RAMEND)     ;initialize stackpointer
        out     SPL, temp01
        ldi     temp01, HIGH(RAMEND)
        out     SPH, temp01

        ;initialize ports
        ldi     temp01, 0xFF            ;Set all pins of port B as output
        out     DDRB, temp01

        ldi     temp01, 0b11111000      ;Set pins PD3-PD7 of Port D as output, PD0-PD2 as input
        out     DDRD, temp01

        ldi     temp01, 0b00000000
        out     DDRC, temp01            ;set port C as input

        ldi     temp01, 0b00011001      ;Activate pullup resistance at PC0, 3, 4 (=forward/reverse switch and jumper)
        out     PORTC, temp01

        ;initialize analog digital converter
        ldi     temp01, 1<<REFS0 | 1<<ADLAR | 1<<MUX0     ;set values for analog digital converter
        out     ADMUX, temp01           ;activate ADC

        ldi     temp01, 1<<ADEN | 1<<ADSC | 1<<ADPS2 | 1<<ADPS1
        out     ADCSRA, temp01          ;start conversion

        ;initialize timer
        ldi     temp01, 1<<CS00 | 1<<CS01    ;divisor 64
        out     TCCR0, temp01

        ldi     temp01, 1<<TOIE0        ;TOIE0: Interrupt at timer overflow
        out     TIMSK, temp01

        ldi     TimerPause, 0

        ldi     ControlBits, 0

        sei

loop:

        ;check hardware jumper status
        mov     temp01, ControlBits
        andi    temp01, 0b11100111
        sbis    PINC, 3               ;skip next operation, if bit 0 is set (1, high, jumper half usage open)
        ori     temp01, 0b00001000
        sbis    PINC, 4               ;skip next operation, if bit 0 is set (1, high, jumper quater usage open)
        ori     temp01, 0b00010000
        mov     ControlBits, temp01


        ;check throttle
        sbic    ADCSRA, ADSC
        rjmp    SkipThrottle
        in      temp01, ADCL        ;read LOW byte
        in      temp02, ADCH        ;read HIGH byte

        ldi     temp01, 0b00000100
        eor     ControlBits, temp01        ;toggle bit 2 (indicator for ADC channel)

        sbrs    ControlBits, 2
        ldi     temp01, 1<<REFS0 | 1<<ADLAR | 1<<MUX0     ;set values for analog digital converter channel 1
        sbrc    ControlBits, 2
        ldi     temp01, 1<<REFS0 | 1<<ADLAR | 1<<MUX1     ;set values for analog digital converter channel 2
        out     ADMUX, temp01           ;activate ADC

        sbi     ADCSRA, ADSC            ;start next conversion


        ldi     temp01, 5
        cpi     temp02, 50
        brlo    SetThrottle

        ldi     temp01, 10
        cpi     temp02, 100
        brlo    SetThrottle

        ldi     temp01, 15
        cpi     temp02, 150
        brlo    SetThrottle

        ldi     temp01, 20

SetThrottle:
        sbrc    ControlBits, 2
        mov     SpeedValue, temp01     ;skipped if bit 3 of ControlBits is 0 (ADC indicated ThrottleValue)
        sbrs    ControlBits, 2
        mov     ThrottleValue, temp01  ;skipped if bit 3 of ControlBits is 1 (ADC indicated SpeedValue)

SkipThrottle:

        ;check forward/reverse switch
        sbic    PINC, 0               ;skip next operation, if bit 0 is cleared (0, low, forward/reverse switch pressed)
        rjmp    ContinueFR01
        ;forward/reverse switch pressed
        sbrs    ControlBits, 1             ;skip next operation if switch was already pressed during last pass
        ldi     SwitchTimer, 200           ;initialize SwitchTimer (switch must be pressed for 200 passes)
        ori     ControlBits, 0b00000010    ;set bit number 1 to 1 (high), leaving other bits unchanged (switch is pressed)
        rjmp    EndFR

ContinueFR01:
        ;forward/reverse switch released
        cpi     SwitchTimer, 0
        brne    ContinueFR02              ;branch if switch wasn't pressed long enough
        sbrs    ControlBits, 1
        rjmp    EndFR                     ;no jump if switch was pressed during previous pass
        ldi     temp01, 0b00000001
        eor     ControlBits, temp01       ;toggle bit 0 (indicator for motor direction)
        sbrc    ControlBits, 0
        sbi     PORTD, 3                  ;set bit number 3 of Port D (control light forward/reverse switch)
        sbrs    ControlBits, 0
        cbi     PORTD, 3                  ;clear bit number 3 of Port D (control light forward/reverse switch)

ContinueFR02:
        andi     ControlBits, 0b11111101  ;set bit number 1 to 0 (low), leaving other bits unchanged
        ldi      SwitchTimer, 0

EndFR:

        ;step motor
        cpi     ThrottleValue, 20
        brne    ContinueStep01
        rcall   Sub_StepMotor01
ContinueStep01:
        cpi     ThrottleValue, 15
        brne    ContinueStep02
        rcall   Sub_StepMotor02
ContinueStep02:
        cpi     ThrottleValue, 10
        brne    ContinueStep05
        rcall   Sub_StepMotor03
ContinueStep05:
        cpi     ThrottleValue, 5
        brne    ContinueStep06
        rcall   Sub_StepMotor04
ContinueStep06:

        rcall   Sub_SetPorts
        rjmp    loop

;++++++++++++++++++++++++++++++++++ Motor step 01 +++++++++++++++++++++++++++++++++++++++++

Sub_StepMotor01:

        ldi     temp01, 0b00000001
        ldi     temp02, 0b00000000

        cpi     StepTimeCount, 10
        brlo    StepMotorContinue01               ;branch if lower
        ldi     temp01, 0b00000010

        cpi     StepTimeCount, 20
        brlo    StepMotorContinue01               ;branch if lower
        ldi     temp01, 0b00000100

        cpi     StepTimeCount, 30
        brlo    StepMotorContinue01               ;branch if lower
        ldi     temp01, 0b00001000

        cpi     StepTimeCount, 40
        brlo    StepMotorContinue01               ;branch if lower
        ldi     temp01, 0b00010000
        ldi     temp02, 0b00000000

        cpi     StepTimeCount, 50
        brlo    StepMotorContinue01               ;branch if lower
        ldi     temp01, 0b00100000
        ldi     temp02, 0b00000000

        cpi     StepTimeCount, 60
        brlo    StepMotorContinue01               ;branch if lower
        ldi     temp01, 0b01000000
        ldi     temp02, 0b00000000

        cpi     StepTimeCount, 70
        brlo    StepMotorContinue01               ;branch if lower
        ldi     temp01, 0b10000000
        ldi     temp02, 0b00000000

        cpi     StepTimeCount, 80
        brlo    StepMotorContinue01               ;branch if lower
        ldi     temp01, 0b00000000
        ldi     temp02, 0b00010000

        cpi     StepTimeCount, 90
        brlo    StepMotorContinue01               ;branch if lower
        ldi     temp01, 0b00000000
        ldi     temp02, 0b00100000

        cpi     StepTimeCount, 100
        brlo    StepMotorContinue01               ;branch if lower
        ldi     temp01, 0b00000000
        ldi     temp02, 0b01000000

        cpi     StepTimeCount, 110
        brlo    StepMotorContinue01               ;branch if lower
        ldi     temp01, 0b00000000
        ldi     temp02, 0b10000000


StepMotorContinue01:
        ret
;---------------------------------- End motor step 01 -------------------------------------

;++++++++++++++++++++++++++++++++++ Motor step 02 +++++++++++++++++++++++++++++++++++++++++

Sub_StepMotor02:

        ldi     temp01, 0b01000001
        ldi     temp02, 0b00000000

        cpi     StepTimeCount, 10
        brlo    StepMotorContinue02               ;branch if lower
        ldi     temp01, 0b10000010

        cpi     StepTimeCount, 20
        brlo    StepMotorContinue02               ;branch if lower
        ldi     temp01, 0b00000100
        ldi     temp02, 0b00010000

        cpi     StepTimeCount, 30
        brlo    StepMotorContinue02               ;branch if lower
        ldi     temp01, 0b00001000
        ldi     temp02, 0b00100000

        cpi     StepTimeCount, 40
        brlo    StepMotorContinue02               ;branch if lower
        ldi     temp01, 0b00010000
        ldi     temp02, 0b01000000

        cpi     StepTimeCount, 50
        brlo    StepMotorContinue02               ;branch if lower
        ldi     temp01, 0b00100000
        ldi     temp02, 0b10000000

        cpi     StepTimeCount, 60
        brlo    StepMotorContinue02               ;branch if lower
        ldi     temp01, 0b01000001
        ldi     temp02, 0b00000000

        cpi     StepTimeCount, 70
        brlo    StepMotorContinue02               ;branch if lower
        ldi     temp01, 0b10000010
        ldi     temp02, 0b00000000

        cpi     StepTimeCount, 80
        brlo    StepMotorContinue02               ;branch if lower
        ldi     temp01, 0b00000100
        ldi     temp02, 0b00010000

        cpi     StepTimeCount, 90
        brlo    StepMotorContinue02               ;branch if lower
        ldi     temp01, 0b00001000
        ldi     temp02, 0b00100000

        cpi     StepTimeCount, 100
        brlo    StepMotorContinue02               ;branch if lower
        ldi     temp01, 0b00010000
        ldi     temp02, 0b01000000

        cpi     StepTimeCount, 110
        brlo    StepMotorContinue02               ;branch if lower
        ldi     temp01, 0b00100000
        ldi     temp02, 0b10000000


StepMotorContinue02:
        ret
;---------------------------------- End motor step 02 -------------------------------------

;++++++++++++++++++++++++++++++++++ Motor step 03 +++++++++++++++++++++++++++++++++++++++++

Sub_StepMotor03:

        ldi     temp01, 0b01001001
        ldi     temp02, 0b00000000

        cpi     StepTimeCount, 10
        brlo    StepMotorContinue03               ;branch if lower
        ldi     temp01, 0b10010010
        ldi     temp02, 0b00000000

        cpi     StepTimeCount, 20
        brlo    StepMotorContinue03               ;branch if lower
        ldi     temp01, 0b00100100
        ldi     temp02, 0b00010000

        cpi     StepTimeCount, 30
        brlo    StepMotorContinue03               ;branch if lower
        ldi     temp01, 0b01001000
        ldi     temp02, 0b00100000

        cpi     StepTimeCount, 40
        brlo    StepMotorContinue03               ;branch if lower
        ldi     temp01, 0b10010000
        ldi     temp02, 0b01000000

        cpi     StepTimeCount, 50
        brlo    StepMotorContinue03               ;branch if lower
        ldi     temp01, 0b00100000
        ldi     temp02, 0b10010000

        cpi     StepTimeCount, 60
        brlo    StepMotorContinue03               ;branch if lower
        ldi     temp01, 0b01000001
        ldi     temp02, 0b00100000

        cpi     StepTimeCount, 70
        brlo    StepMotorContinue03               ;branch if lower
        ldi     temp01, 0b10000010
        ldi     temp02, 0b01000000

        cpi     StepTimeCount, 80
        brlo    StepMotorContinue03               ;branch if lower
        ldi     temp01, 0b00000100
        ldi     temp02, 0b10010000

        cpi     StepTimeCount, 90
        brlo    StepMotorContinue03               ;branch if lower
        ldi     temp01, 0b00001001
        ldi     temp02, 0b00100000

        cpi     StepTimeCount, 100
        brlo    StepMotorContinue03               ;branch if lower
        ldi     temp01, 0b00010010
        ldi     temp02, 0b01000000

        cpi     StepTimeCount, 110
        brlo    StepMotorContinue03               ;branch if lower
        ldi     temp01, 0b00100100
        ldi     temp02, 0b10000000


StepMotorContinue03:
        ret
;---------------------------------- End motor step 03 -------------------------------------


;++++++++++++++++++++++++++++++++++ Motor step 04 +++++++++++++++++++++++++++++++++++++++++

Sub_StepMotor04:

        ldi     temp01, 0b01001001
        ldi     temp02, 0b00100000

        cpi     StepTimeCount, 10
        brlo    StepMotorContinue04               ;branch if lower
        ldi     temp01, 0b10010010
        ldi     temp02, 0b01000000

        cpi     StepTimeCount, 20
        brlo    StepMotorContinue04               ;branch if lower
        ldi     temp01, 0b00100100
        ldi     temp02, 0b10010000

        cpi     StepTimeCount, 30
        brlo    StepMotorContinue04               ;branch if lower
        ldi     temp01, 0b01001001
        ldi     temp02, 0b00100000

        cpi     StepTimeCount, 40
        brlo    StepMotorContinue04               ;branch if lower
        ldi     temp01, 0b10010010
        ldi     temp02, 0b01000000

        cpi     StepTimeCount, 50
        brlo    StepMotorContinue04               ;branch if lower
        ldi     temp01, 0b00100100
        ldi     temp02, 0b10010000

        cpi     StepTimeCount, 60
        brlo    StepMotorContinue04               ;branch if lower
        ldi     temp01, 0b01001001
        ldi     temp02, 0b00100000

        cpi     StepTimeCount, 70
        brlo    StepMotorContinue04               ;branch if lower
        ldi     temp01, 0b10010010
        ldi     temp02, 0b01000000

        cpi     StepTimeCount, 80
        brlo    StepMotorContinue04               ;branch if lower
        ldi     temp01, 0b00100100
        ldi     temp02, 0b10010000

        cpi     StepTimeCount, 90
        brlo    StepMotorContinue04               ;branch if lower
        ldi     temp01, 0b01001001
        ldi     temp02, 0b00100000

        cpi     StepTimeCount, 100
        brlo    StepMotorContinue04               ;branch if lower
        ldi     temp01, 0b10010010
        ldi     temp02, 0b01000000

        cpi     StepTimeCount, 110
        brlo    StepMotorContinue04               ;branch if lower
        ldi     temp01, 0b00100100
        ldi     temp02, 0b10010000


StepMotorContinue04:
        ret
;---------------------------------- End motor step 04 -------------------------------------

;++++++++++++++++++++++++++++++++++ set ports +++++++++++++++++++++++++++++++++++++++++++++
Sub_SetPorts:                               ;set output pins for motor rotation

        sbrc    ControlBits, 0              ;skip next operation if bit 0 is cleared (forward/reverse indicator)
        ori     temp02, 0b00001000          ;set red LED to ON

        out     PORTB, temp01
        out     PORTD, temp02

        ret

;---------------------------------- End set ports -----------------------------------------

;++++++++++++++++++++++++++++++++++ Timer interrupt +++++++++++++++++++++++++++++++++++++++

timer0_overflow:                      ;Timer 0 Overflow Handler
        push    temp01                ;save temp01
        in      temp01, SREG          ;save status register
        push    temp01

        cpi     SwitchTimer, 0
        breq    Skip00
        dec     SwitchTimer

Skip00:
        inc     TimerPause
        cp      TimerPause, SpeedValue
        brlo    TimerContinue
        ldi     TimerPause, 0
        sbrc    ControlBits, 0        ;skip next operation if bit 0 is cleared (=forward direction)
        dec     StepTimeCount         ;decrease for reverse step
        sbrs    ControlBits, 0        ;skip next operation if bit 0 is set (=reverse direction)
        inc     StepTimeCount         ;increase for forward step
        cpi     StepTimeCount, 255    ;subtract underflow
        brne    Skip01
        ldi     StepTimeCount, 119
        sbrc    ControlBits, 3
        ldi     StepTimeCount, 59
        sbrc    ControlBits, 4
        ldi     StepTimeCount, 29

Skip01:
        sbrc    ControlBits, 0
        rjmp    Skip04
        cpi     StepTimeCount, 120    ;count til 119
        brlo    Skip02
        clr     StepTimeCount

Skip02:
        cpi     StepTimeCount, 60    ;count til 59 when half usage is set
        brlo    Skip03
        sbrc    ControlBits, 3
        clr     StepTimeCount

Skip03:
        cpi     StepTimeCount, 30    ;count til 29 when half usage is set
        brlo    Skip04
        sbrc    ControlBits, 4
        clr     StepTimeCount

Skip04:

TimerContinue:

        pop     temp01
        out     SREG, temp01          ;restore status register
        pop     temp01                ;restore temp01

        reti